package com.minemaarten.signals.capabilities.destinationproviders;

import java.util.List;
import java.util.regex.Pattern;

import net.minecraft.client.resources.I18n;
import net.minecraft.entity.item.EntityMinecart;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.inventory.IInventory;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.ResourceLocation;
import net.minecraftforge.common.capabilities.Capability;
import net.minecraftforge.common.capabilities.CapabilityInject;
import net.minecraftforge.items.IItemHandler;
import net.minecraftforge.oredict.OreDictionary;

import com.minemaarten.signals.api.Signals;
import com.minemaarten.signals.api.access.IDestinationProviderItems;
import com.minemaarten.signals.network.GuiSynced;
import com.minemaarten.signals.proxy.CommonProxy.EnumGuiId;
import com.minemaarten.signals.tileentity.IGUIButtonSensitive;

@Signals
public class DestinationProviderItems implements IDestinationProviderItems, IGUIButtonSensitive{
    @CapabilityInject(IItemHandler.class)
    private static Capability<IItemHandler> ITEM_HANDLER;

    @GuiSynced
    public boolean blacklist, checkDamage = true, checkNBT, checkModSimilarity, checkOreDictionary;

    @Override
    public boolean isTileEntityApplicable(TileEntity te){
        return te instanceof IInventory;
    }

    @Override
    public boolean isCartApplicable(TileEntity te, EntityMinecart cart, Pattern destinationRegex){
        IItemHandler cap = cart.getCapability(ITEM_HANDLER, null);
        if(cap != null && destinationRegex.matcher("ITEM").matches()) {
            IInventory inv = (IInventory)te;
            for(int cartSlot = 0; cartSlot < cap.getSlots(); cartSlot++) {
                ItemStack cartStack = cap.getStackInSlot(cartSlot);
                if(!cartStack.func_190926_b()) {
                    if(isStackApplicable(cartStack, inv)) return true;
                }
            }
        }
        return false;
    }

    public boolean isStackApplicable(ItemStack cartStack, IInventory inv){
        for(int teSlot = 0; teSlot < inv.func_70302_i_(); teSlot++) {
            ItemStack teStack = inv.func_70301_a(teSlot);
            if(!teStack.func_190926_b()) {
                if(blacklist != areStacksEqual(cartStack, teStack, checkDamage, checkNBT, checkOreDictionary, checkModSimilarity)) return true;
            }
        }
        return false;
    }

    public static boolean areStacksEqual(ItemStack stack1, ItemStack stack2, boolean checkMeta, boolean checkNBT, boolean checkOreDict, boolean checkModSimilarity){
        if(stack1.func_190926_b() && stack2.func_190926_b()) return true;
        if(stack1.func_190926_b() && !stack2.func_190926_b() || !stack1.func_190926_b() && stack2.func_190926_b()) return false;

        if(checkModSimilarity) {
            ResourceLocation id1 = Item.field_150901_e.func_177774_c(stack1.func_77973_b());
            ResourceLocation id2 = Item.field_150901_e.func_177774_c(stack2.func_77973_b());
            return id1 != null && id2 != null && id1.func_110624_b().equals(id2.func_110624_b());
        }
        if(checkOreDict) {
            return isSameOreDictStack(stack1, stack2);
        }

        if(stack1.func_77973_b() != stack2.func_77973_b()) return false;

        boolean metaSame = stack1.func_77952_i() == stack2.func_77952_i();
        boolean nbtSame = stack1.func_77942_o() ? stack1.func_77978_p().equals(stack2.func_77978_p()) : !stack2.func_77942_o();

        return (!checkMeta || metaSame) && (!checkNBT || nbtSame);
    }

    public static boolean isSameOreDictStack(ItemStack stack1, ItemStack stack2){
        int[] oredictIds = OreDictionary.getOreIDs(stack1);
        for(int oredictId : oredictIds) {
            List<ItemStack> oreDictStacks = OreDictionary.getOres(OreDictionary.getOreName(oredictId));
            for(ItemStack oreDictStack : oreDictStacks) {
                if(OreDictionary.itemMatches(oreDictStack, stack2, false)) {
                    return true;
                }
            }
        }
        return false;
    }

    @Override
    public boolean hasGui(TileEntity te){
        return true;
    }

    @Override
    public void openGui(TileEntity te, EntityPlayer player){
        player.openGui(com.minemaarten.signals.Signals.instance, EnumGuiId.ITEM_HANDLER_DESTINATION.ordinal(), player.field_70170_p, te.func_174877_v().func_177958_n(), te.func_174877_v().func_177956_o(), te.func_174877_v().func_177952_p());
    }

    @Override
    public void handleGUIButtonPress(EntityPlayer player, int... data){
        switch(data[0]){
            case 0:
                checkDamage = !checkDamage;
                break;
            case 1:
                checkNBT = !checkNBT;
                break;
            case 2:
                checkModSimilarity = !checkModSimilarity;
                break;
            case 3:
                checkOreDictionary = !checkOreDictionary;
                break;
            case 4:
                blacklist = !blacklist;
        }
    }

    @Override
    public String getLocalizedName(){
        return I18n.func_135052_a("signals.gui.destination_provider.items");
    }

    @Override
    public void writeToNBT(NBTTagCompound tag){
        //Invert checkDamage, as that one is true by default, and save disk space for all inventories not used
        byte packedData = (byte)((blacklist ? 16 : 0) + (checkDamage ? 0 : 8) + (checkNBT ? 4 : 0) + (checkModSimilarity ? 2 : 0) + (checkOreDictionary ? 1 : 0));
        if(packedData > 0) tag.func_74774_a("filterParams", packedData);
    }

    @Override
    public void readFromNBT(NBTTagCompound tag){
        byte packetData = tag.func_74771_c("filterParams");
        blacklist = (packetData & 16) > 0;
        checkDamage = (packetData & 8) == 0;
        checkNBT = (packetData & 4) > 0;
        checkModSimilarity = (packetData & 2) > 0;
        checkOreDictionary = (packetData & 1) > 0;
    }

    @Override
    public void useBlacklist(boolean useBlacklist){
        blacklist = useBlacklist;
    }

    @Override
    public boolean isUsingBlacklist(){
        return blacklist;
    }

    @Override
    public void checkDamage(boolean checkDamage){
        this.checkDamage = checkDamage;
    }

    @Override
    public boolean isCheckingDamage(){
        return checkDamage;
    }

    @Override
    public void checkNBT(boolean checkNBT){
        this.checkNBT = checkNBT;
    }

    @Override
    public boolean isCheckingNBT(){
        return checkNBT;
    }

    @Override
    public void checkModSimilarity(boolean checkModSimilarity){
        this.checkModSimilarity = checkModSimilarity;
    }

    @Override
    public boolean isCheckingModSimilarity(){
        return checkModSimilarity;
    }

    @Override
    public void checkOreDictionary(boolean checkOreDictionary){
        this.checkOreDictionary = checkOreDictionary;
    }

    @Override
    public boolean isCheckingOreDictionary(){
        return checkOreDictionary;
    }
}
